"""Functions that deals with dummy samples"""
from cg.apps.demultiplex.sample_sheet.models import (
    FlowCellSample,
    FlowCellSampleBcl2Fastq,
    FlowCellSampleDragen,
)
from cg.constants.demultiplexing import BclConverter
from typing import Union


def dummy_sample_name(sample_name: str) -> str:
    """Convert a string to a dummy sample name replacing spaces and parentheses with dashes."""
    return sample_name.replace(" ", "-").replace("(", "-").replace(")", "-")


def dummy_sample(
    flow_cell_id: str, dummy_index: str, lane: int, name: str, bcl_converter: str
) -> Union[FlowCellSampleBcl2Fastq, FlowCellSampleDragen]:
    """Constructs and returns a dummy sample in Novaseq sample sheet format."""
    flowcell_sample = {
        BclConverter.BCL2FASTQ.value: FlowCellSampleBcl2Fastq,
        BclConverter.DRAGEN.value: FlowCellSampleDragen,
    }
    return flowcell_sample[bcl_converter](
        flowcell_id=flow_cell_id,
        lane=lane,
        sample_id=dummy_sample_name(sample_name=name),
        index=dummy_index,
        sample_name="indexcheck",
        project="indexcheck",
    )



from cg.apps.sequencing_metrics_parser.parsers.bcl2fastq_to_sequencing_statistics import (
    calculate_average_quality_score_for_sample_in_lane,
    calculate_q30_ratio_for_sample_in_lane,
)
from cg.apps.sequencing_metrics_parser.models.bcl2fastq_metrics import ConversionResult, DemuxResult


def test_calculate_average_quality_score(conversion_result: ConversionResult):
    """ "Test calculating the average quality score from a conversion result."""
    # GIVEN a conversion result with a known average quality score
    for read in conversion_result.demux_results[0].read_metrics:
        read.quality_score_sum = 100
        read.yield_ = 100

    # WHEN calculating the average quality score
    average_quality_score: float = calculate_average_quality_score_for_sample_in_lane(
        demux_result=conversion_result.demux_results[0]
    )

    # THEN the average quality score should be the sum of the quality scores for all reads in the sample
    # divided by  the total yield for the sample
    assert average_quality_score == 1.0


def test_calculate_q30_ratio(conversion_result: ConversionResult):
    """Test calculating the q30 ratio from a conversion result."""
    # GIVEN a conversion result with a known q30 ratio
    for read in conversion_result.demux_results[0].read_metrics:
        read.yield_q30 = 100
        read.yield_ = 100

    # WHEN calculating the q30 ratio
    q30_ratio: float = calculate_q30_ratio_for_sample_in_lane(
        demux_result=conversion_result.demux_results[0]
    )

    # THEN the q30 ratio should be the sum of the q30 bases for all reads in the sample
    # divided by the total yield for the sample
    assert q30_ratio == 1.0



import logging
from pydantic import BaseModel, Field
from typing import List

from cg.constants.constants import GenomeVersion
from cg.constants.demultiplexing import SampleSheetHeaderColumnNames

LOG = logging.getLogger(__name__)


class FlowCellSample(BaseModel):
    """This model is used when parsing/validating existing sample sheets."""

    flowcell_id: str = Field(..., alias=SampleSheetHeaderColumnNames.FLOW_CELL_ID.value)
    lane: int = Field(..., alias=SampleSheetHeaderColumnNames.LANE.value)
    sample_id: str
    sample_ref: str = Field(GenomeVersion.hg19.value, alias="SampleRef")
    index: str = Field(..., alias="index")
    index2: str = ""
    sample_name: str = Field(..., alias="SampleName")
    control: str = Field("N", alias="Control")
    recipe: str = Field("R1", alias="Recipe")
    operator: str = Field("script", alias="Operator")
    project: str = Field(..., alias="Project")

    class Config:
        allow_population_by_field_name = True


class FlowCellSampleBcl2Fastq(FlowCellSample):
    sample_id: str = Field(..., alias="SampleID")
    project: str = Field(..., alias="Project")


class FlowCellSampleDragen(FlowCellSample):
    sample_id: str = Field(..., alias="Sample_ID")
    project: str = Field(..., alias="Sample_Project")


class SampleSheet(BaseModel):
    samples: List[FlowCellSample]


class SampleSheetBcl2Fastq(SampleSheet):
    samples: List[FlowCellSampleBcl2Fastq]


class SampleSheetDragen(SampleSheet):
    samples: List[FlowCellSampleDragen]
